/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.itvsh.kop.common.logging;

import org.aspectj.lang.JoinPoint;
import org.aspectj.lang.annotation.AfterReturning;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Before;
import org.springframework.stereotype.Component;

@Aspect
@Component
public class KopLoggingAspect extends AspectPointcuts {

	@Before("anyPublicServiceMethod()")
	public void onServiceMethod(JoinPoint joinPoint) {
		AspectLoggingUtils.log(joinPoint);
	}

	@AfterThrowing(pointcut = "anyPublicServiceMethod()", throwing = "ex")
	public void afterExceptionInServiceMethod(JoinPoint joinPoint, Exception ex) {
		AspectLoggingUtils.logException(joinPoint, ex);
	}

	@AfterReturning(pointcut = "anyPublicServiceMethod()", returning = "returnValue")
	public void afterServiceMethod(JoinPoint joinPoint, Object returnValue) {
		AspectLoggingUtils.logReturnValue(joinPoint, returnValue);
	}
}
